package cn.jyjz.xiaoyao.common.redis.cache.ehcache;

import java.io.Serializable;
import net.sf.ehcache.Cache;
import net.sf.ehcache.config.CacheConfiguration;
import net.sf.ehcache.config.CacheConfiguration.CacheEventListenerFactoryConfiguration;
import net.sf.ehcache.config.TerracottaConfiguration;
import org.apache.commons.lang3.StringUtils;

/**
 * Created by zhangyong on 2018/4/18.
 */
public class CacheUtileBase implements Serializable {

    private static final long serialVersionUID = 2988213828504611915L;


    /**
     * 创建缓存
     * @param url terracotta服务地址
     * @param cachename 缓存名称
     * @param saveTime 缓存的存过时间 -1：表示表示不起作用
     * @param validTime 活动时间 -1：表示表示不起作用
     * @param eternal 是否永久缓存
     * @return 返回缓存对象
     */
    public static synchronized Cache initCache(String url,long saveTime, String cachename, long validTime,boolean eternal){

        //存放数量
        int maxEntriesLocalHeap = 1000000;
        CacheConfiguration cacheConfiguration = new CacheConfiguration(cachename, maxEntriesLocalHeap);

        //url不为空，说明使用terracotta缓存服务
        if(StringUtils.isNotBlank(url)){

            TerracottaConfiguration terracottaConfiguration = new TerracottaConfiguration();
            terracottaConfiguration.setClustered(true);
            cacheConfiguration.terracotta(terracottaConfiguration);
        }

        //是否为永久缓存
        cacheConfiguration.setEternal(eternal);
        //缓存清除策略有：LRU（最近最少使用，默认策略）、FIFO（先进先出）、LFU（最少访问次数
        cacheConfiguration.setMemoryStoreEvictionPolicy("LRU");

        cacheConfiguration.setTransactionalMode("off");

        CacheEventListenerFactoryConfiguration celc =
            new CacheEventListenerFactoryConfiguration();

        celc.setClass("cn.jyjz.xiaoyao.common.redis.cache.ehcache.listener.CacheEventListener");
        //celc.setProperties("cache=mychache");
        cacheConfiguration.addCacheEventListenerFactory(celc);
        //cacheConfiguration.addBootstrapCacheLoaderFactory(new MyCacheManagerEventListener(null));

        /**
         *  其中主要记录的是timeToLiveSeconds和timeToIdleSeconds；因为此俩容易搞混淆：

         timeToLiveSeconds=x：缓存自创建日期起至失效时的间隔时间x；

         timeToIdleSeconds=y：缓存创建以后，最后一次访问缓存的日期至失效之时的时间间隔y；


         如果仅有 timeToLiveSeconds 那么 自创建时间开始 间隔x后缓存失效；

         如果没有timeToLiveSeconds 那么自最后一次访问缓存 间隔y后 缓存失效；

         如果既有timeToLiveSeconds 也有 timeToIdleSeconds 那么取最小数算作间隔时间；min(x,y);； 经过测试其计算原则是：若自创建缓存后一直都没有访问缓存，那么间隔x后失效，若自创建缓存后有N次访问缓存，那么计算（最后一次访问缓存时间+y ） 即：按照timeToIdleSeconds计算，但总存活时间不超过 y;举个例子：

         timeToIdleSeconds=120；

         timeToLiveSeconds=180；

         上面的表示此缓存最多可以存活3分钟，如果期间超过2分钟未访问 那么此缓存失效！

         1）单独设置 timeToLiveSeconds：

         该对象的超期时间 = 初始创建时间 + timeToLiveSeconds

         因为初始创建时间是固定的，因此不管这个对象在有效期内被命中了多少次，一旦满足超期条件，该对象将被移除。

         2）单独设置 timeToIdleSeconds：

         该对象的超时时间 = 最近访问时间 + timeToIdleSeconds

         注意与上面的区别：不再根据创建时间，而是根据最近访问时间来确定超期时间

         所以这是一种动态的超期模式，即使这个参数设置为1（秒），只要保证每秒内都能get一次，那么对象也将永远不会超期。

         3）分别设置 timeToLiveSeconds 及 timeToIdleSeconds ：

         那么将分别计算以上两种模式的超期时间，会得出两个结果，再从两个结果里找到最小的一个做为超期时间，相当于“严苛模式”

         但事实上，创建时间肯定会小于最近访问时间，那如果两者都设置同样的参数值，相当于 timeToIdleSeconds 永远也不会起到作用。

         如果设置不同的参数值，根据具体的业务需求，可能会出现一些意料之中或者意料之外的情况。
         */
        //缓存的存放时间
        //cacheConfiguration.setTimeToLiveSeconds(saveTime);

        //不为-1时，标示活动时间不起作用
//        if(validTime != -1){
//            if(validTime >= saveTime){
//                cacheConfiguration.setTimeToIdleSeconds(saveTime - 10);
//            }else{
//                cacheConfiguration.setTimeToIdleSeconds(validTime);
//            }
//        }else{
//            cacheConfiguration.setTimeToIdleSeconds(saveTime - 10);
//        }
        if(!eternal){
            if(validTime > -1 && saveTime > -1){
                if(validTime >= saveTime){
                    cacheConfiguration.setTimeToIdleSeconds(validTime);
                }else{
                    cacheConfiguration.setTimeToIdleSeconds(saveTime);
                }
            }
            else if(saveTime != -1){
                cacheConfiguration.setTimeToLiveSeconds(saveTime);
            }
            else if(validTime != -1){
                cacheConfiguration.setTimeToLiveSeconds(saveTime+1);
                cacheConfiguration.setTimeToIdleSeconds(validTime);
            }else{
                cacheConfiguration.setTimeToIdleSeconds(1800);
            }
        }

        //是否使用磁盘
        cacheConfiguration.setOverflowToDisk(false);
        //硬盘中最大缓存对象数，若是0表示无穷大
        cacheConfiguration.setMaxElementsOnDisk(0);

        //初始化缓存对象
        Cache cache = new Cache(cacheConfiguration);

        return cache;
    }
}
