package cn.jyjz.xiaoyao.common.redis.cache.redis;


import cn.jyjz.xiaoyao.common.redis.cache.ICacheManager;
import cn.jyjz.xiaoyao.common.redis.config.CacheProperties;
import com.alibaba.fastjson.JSONObject;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import java.util.concurrent.TimeUnit;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.redis.core.RedisCallback;

/**
 * Created by zhangyong on 2018/9/14.
 */
public class RedisManagerImpl implements ICacheManager {

    private Logger log = LoggerFactory.getLogger(this.getClass());

    CacheProperties cacheProperties;

    private RedisUtils redisUtils;

    public RedisManagerImpl(RedisUtils redisUtils,CacheProperties cacheProperties){
        this.redisUtils = redisUtils;
        this.cacheProperties = cacheProperties;
    }

    /**
     * 在redis中，粗放的key，使用的是cacheName_key
     * 获得当前的获取value时，需要判断cacheName_keytime是否存在，用户判断是否要增加当前数据在缓存中的时候加否要增加
     * @param cacheName 缓存名称
     * @param key 缓存key，目前用在MAP中
     * @param <T>
     * @return
     */
    @Override
    public <T> T get(String cacheName, String key) {


        //组装cacheName与key的名称
        String redisObjectKey = cacheProperties.getPrefix() + "{" + cacheName + cacheProperties.getCacheNamePrix() + "}" + key;

        //过期时间的主键
        String keyTime = redisObjectKey + cacheProperties.getCacheNamePrix() + cacheProperties.getCacheNameTimePrix();

        List<String> keys = new ArrayList<>();
        keys.add(redisObjectKey);
        keys.add(keyTime);


        //通过lua表达式获得缓存中的数据
        Object objectvalue = redisUtils.executeLua(keys);

        if(null == objectvalue){
            return null;
        }
        List list = (List) objectvalue;
        list.removeAll(Collections.singleton(null));
        //list.remove(null);
        //判断数据是否存在
        if(null != list && list.size() > 0){
            //反序列化对象
            Object values =  SerializeUtil.objectDeserialization(list.get(0).toString());
            //对象不为空返回对象
            if(null != values){
                return (T) values;
            }

        }

        return null;

    }

    @Override
    public <T> List<T> get(String cacheName) {
        //缓存名称，增加前缀，防止多系统情况下出现主键覆盖问题
        cacheName = cacheProperties.getPrefix() + "{" + cacheName + cacheProperties.getCacheNamePrix() + "}";


        //根据cacheName获得集合信息
        Set<String> set = redisUtils.getKeys(cacheName + "*");

        List<T> list = new ArrayList<T>();
        //遍历redis中的集合
        for(String keyvalue:set){

            //list.add((T)redisTempKey);
            //判断cacheName中的集合是否存在值
            if(redisUtils.exists(keyvalue)){
                Object values = redisUtils.get(keyvalue);

                Object returnvalues = SerializeUtil.objectDeserialization(values.toString());

                if(null != values){
                    list.add((T) returnvalues);
                }
            }
        }

        return list;
    }

    @Override
    public void put(String cacheName, String key, Object value) {
        this.put(cacheName,key,value,-1,-1);
    }

    @Override
    public void put(String cacheName, String key, Object value, long validTime) {
        this.put(cacheName,key,value,-1,validTime);
    }

    /**
     *
     * 添加缓存
     * 1、保存cacheName_key作为redis的主键存放value
     * 2、保存cacheName作为redis的主键存放对应key，后续好知道我这种类型存多少数据
     * 3、根据validTime时间使用cacheName_keytime作为redis的主键存放value在缓存中存活的时间，用于后续获取时是否要增加对应的key的存活时间
     * 4、saveTime、validTime都小于等于零，说明当前数据是要永久存活在缓存中
     * 添加一个缓冲数据
     * @param key 缓存key
     * @param value 缓冲的缓存数据
     * @param saveTime 存活时间，单位是秒
     * @param validTime 超时时间，存活时间，单位是秒
     */
    @Override
    public void put(String cacheName, String key, Object value, long saveTime, long validTime) {

        //cacheName和key组合为value的key值
        String redisObjectKey = cacheProperties.getPrefix() + "{" + cacheName + cacheProperties.getCacheNamePrix() + "}" + key ;

        String valueString = SerializeUtil.objectSerialiable(value);

        //如果存活
        if(saveTime > 0){
            redisUtils.set(redisObjectKey,valueString,saveTime, TimeUnit.SECONDS);

        }else if(validTime> 0){//在缓存中的存活时间，如果在存活时间内被使用过（get），存活时间重新初始化

            //当前key的存活时间对应的 临时key
            String keyTime = redisObjectKey + cacheProperties.getCacheNamePrix() + cacheProperties.getCacheNameTimePrix();

            List<String> keys = new ArrayList<String>();
            keys.add(redisObjectKey);
            keys.add(keyTime);

            /**
             * 用Mpa设置Lua的ARGV[1]
             */
            Map<String,Object> argvMap = new HashMap<String,Object>();
            argvMap.put("valuemodel",valueString);
            argvMap.put("times",validTime);

            long start = System.currentTimeMillis();

            redisUtils.executeLua(keys,valueString,validTime + "");

        }else{
            //添加key和value的关系
            redisUtils.set(redisObjectKey,valueString);
        }
    }

    /**
     * 删除一个缓存数据问题
     * 1、删除cacheName_key作为redis的主键的数据
     * 2、删除cacheName集合中的数据
     * 3、删除cacheName_keytime的数据
     * @param key 缓存key
     */
    @Override
    public void remove(String cacheName, String key) {

        //cacheName和key组合为value的key值
        String redisObjectKey = cacheProperties.getPrefix() + "{" + cacheName + cacheProperties.getCacheNamePrix() + "}" + key;

        //过期时间的主键
        String keyTime = redisObjectKey + cacheProperties.getCacheNamePrix() + cacheProperties.getCacheNameTimePrix();

        List<String> keys = new ArrayList<String>();
        keys.add(keyTime);
        keys.add(redisObjectKey);


        redisUtils.executeLuaDel(keys);
        //redisUtils.executeLua(luaDel,keys,keys);

    }

    /**
     * 删除一个缓存数据问题
     * 1、删除cacheName_key作为redis的主键的数据
     * 2、删除cacheName集合中所有key的数据
     * 3、删除cacheName_keytime的数据
     *
     * @param cacheName 缓存类型
     */
    @Override
    public void removeAll(String cacheName) {
        //缓存名称，增加前缀，防止多系统情况下出现主键覆盖问题
        cacheName = cacheProperties.getPrefix() + "{" + cacheName + cacheProperties.getCacheNamePrix()  + "}";

        //根据cacheName获得集合信息
        Set<String> set = redisUtils.getKeys(cacheName + "*");

        List<String> keys = new ArrayList<>(set);

        redisUtils.executeLuaDel(keys);
    }

    @Override
    public JSONObject getCacheInfo(){
        JSONObject obj = new JSONObject();

        // 获取redis缓存完整信息
        Properties info = (Properties) redisUtils.getRedisTemplate().execute((RedisCallback<Object>) connection -> connection.info());


        long used_memory = Long.parseLong(info.getProperty("used_memory"));
        long total_system_memory = Long.parseLong(info.getProperty("total_system_memory"));


        obj.put("cachename","redis");
        obj.put("memory",(used_memory/1024/1024) + "MB / " + (total_system_memory/1024/1024) + "MB");


        return obj;
    }
}
